@@ -18,7 +18,7 @@ from django.utils.safestring import mark_safe
 from django.utils.text import truncate_words
 from django.utils.html import strip_tags
 from django.utils.functional import curry
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ugettext as _
 from django.db.models.signals import post_save
 
 from marcus import utils
@@ -415,5 +415,6 @@ post_save.connect(update_tag_and_category_counts, sender=Article)
 def notify_followers(sender, instance, created, **kwargs):
     if instance.approved:
         utils.notify_comment_followers(instance)
-    utils.notify_comment_managers(instance)
+    if not instance.spam_status:
+        utils.notify_comment_managers(instance)
 post_save.connect(notify_followers, sender=Comment)
